#include "GameTypes.h"
#include "GameAPI.h"
#include <string>
#include <algorithm>

/*** NiTPointerMap ***/

TESForm * NiTPointerMap::Lookup(UInt32 key)
{
	for(Entry * traverse = m_buckets[key % m_numBuckets]; traverse; traverse = traverse->next)
		if(traverse->key == key)
			return traverse->data;
	
	return NULL;
}

TESForm * NiTPointerMap::Iterator::Get(void)
{
	if(m_entry)
		return m_entry->data;

	return NULL;
}

bool NiTPointerMap::Iterator::Next(void)
{
	if(m_entry)
		m_entry = m_entry->next;

	while(!m_entry && (m_bucket < m_table->m_numBuckets))
	{
		m_bucket++;

		m_entry = m_table->m_buckets[m_bucket];
	}

	return m_entry != NULL;
}

bool NiTPointerMap::Iterator::Done(void)
{
	return m_entry == NULL;
}

void NiTPointerMap::Iterator::FindValid(void)
{
	// validate bucket
	if(m_bucket >= m_table->m_numBuckets) return;

	// get bucket
	m_entry = m_table->m_buckets[m_bucket];

	// find non-empty bucket
	while(!m_entry && (m_bucket < m_table->m_numBuckets))
	{
		m_bucket++;

		m_entry = m_table->m_buckets[m_bucket];
	}
}

#if OBLIVION_VERSION == OBLIVION_VERSION_1_1
NiTPointerMap * g_formTable = (NiTPointerMap *)0x00AEDE44;
#elif OBLIVION_VERSION == OBLIVION_VERSION_1_2
NiTPointerMap * g_formTable = (NiTPointerMap *)0x00B0613C;
#elif OBLIVION_VERSION == OBLIVION_VERSION_1_2_416
NiTPointerMap * g_formTable = (NiTPointerMap *)0x00B0613C;
#else
#error unsupported version of oblivion
#endif

/*** String ***/

bool String::Set(const char * src)
{
	UInt32	srcLen = strlen(src);

	// realloc if needed
	if(srcLen > m_bufLen)
	{
		FormHeap_Free(m_data);
		m_data = (char *)FormHeap_Allocate(srcLen + 1);
		m_bufLen = m_data ? srcLen : 0;
	}

	if(m_data)
	{
		strcpy_s(m_data, m_bufLen + 1, src);
		m_dataLen = srcLen;
	}
	else
	{
		m_dataLen = 0;
	}

	return m_data != NULL;
}

bool ci_equal(char ch1, char ch2)
{
	return tolower((unsigned char)ch1) == tolower((unsigned char)ch2);
}

bool String::Includes(const char *toFind) const
{
	std::string curr(m_data, m_dataLen);
	std::string str2(toFind);
	std::string::iterator currEnd = curr.end();
	return (std::search(curr.begin(), currEnd, str2.begin(), str2.end(), ci_equal) != currEnd);
}

bool String::Replace(const char *_toReplace, const char *_replaceWith)
{
	std::string curr(m_data, m_dataLen);
	std::string toReplace(_toReplace);

	std::string::iterator currBegin = curr.begin();
	std::string::iterator currEnd = curr.end();
	std::string::iterator replaceIt = std::search(currBegin, currEnd, toReplace.begin(), toReplace.end(), ci_equal);
	if (replaceIt != currEnd) {
		std::string replaceWith(_replaceWith);
		// we found the substring, now we need to do the modification
		std::string::size_type replacePos = distance(currBegin, replaceIt);
		curr.replace(replacePos, toReplace.size(), replaceWith);
		Set(curr.c_str());
		return true;
	}
	return false;
}